import 'package:account_kit/entities/accessToken/accessToken.dart';
import 'package:account_kit/entities/user/user.dart';
import 'package:account_kit/redux/accessToken/accessTokenActions.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:built_collection/built_collection.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:local_auth/local_auth.dart';
import 'package:puma_network/redux/network/networkAction.dart';
import 'package:puma_ui/ui/atoms/pumaButton.dart';
import 'package:redux/redux.dart';

class SettingsContainer<T extends VirtualAccountState> extends StatefulWidget {
  SettingsContainer();

  @override
  _SettingsContainerState createState() => _SettingsContainerState<T>();
}

class _SettingsContainerState<T extends VirtualAccountState>
    extends State<SettingsContainer> {
  final String title = "Settings";
  final LocalAuthentication auth = LocalAuthentication();
  bool _biometricEnabled = false;
  List<BiometricType>? _availableBiometrics;
  bool _isAuthenticating = false;
  bool _canCheckBiometrics = false;

  void initState() {
    _getAvailableBiometrics();
  }

  Future<void> _checkBiometrics() async {
    bool canCheckBiometrics = false;
    try {
      canCheckBiometrics = await auth.canCheckBiometrics;
    } on PlatformException catch (e) {
      print(e);
    }
    if (!mounted) return;

    setState(() {
      _canCheckBiometrics = canCheckBiometrics;
    });
  }

  Future<void> _getAvailableBiometrics() async {
    List<BiometricType>? availableBiometrics;
    try {
      availableBiometrics = await auth.getAvailableBiometrics();
    } on PlatformException catch (e) {
      print(e);
    }
    if (!mounted) return;

    setState(() {
      _availableBiometrics = availableBiometrics;
    });
  }

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      onInitialBuild: (_ViewModel vm) =>
          _biometricEnabled = vm.biometricAccessToken != null,
      converter: (Store<T> store) => _ViewModel.create(store),
      builder: (BuildContext context, _ViewModel viewModel) =>
          _createBody(context, viewModel),
    );
  }

  getBiometricIcon(BiometricType type) {
    var icon;
    switch (type) {
      case BiometricType.face:
        icon = Image.asset(
          'lib/assets/faceid.png',
          package: "account_kit",
          width: 25,
        );
        break;
      case BiometricType.fingerprint:
        icon = Image.asset(
          'lib/assets/fingerprint.png',
          package: "account_kit",
          width: 25,
        );
        break;
      case BiometricType.iris:
        icon = Image.asset(
          'lib/assets/iris.png',
          package: "account_kit",
          width: 25,
        );
        break;
      default:
        icon = Image.asset(
          'lib/assets/fingerprint.png',
          package: "account_kit",
          width: 25,
        );
    }
    return icon;
  }

  getBiometricLabel(BiometricType type) {
    var label;
    switch (type) {
      case BiometricType.face:
        label = "Face ID";
        break;
      case BiometricType.fingerprint:
        label = "Touch ID";
        break;
      case BiometricType.iris:
        label = "Iris ID";
        break;
      default:
        label = "Biometric";
    }
    return label;
  }

  Future<bool> _authenticate() async {
    bool authenticated = false;
    try {
      setState(() {
        _isAuthenticating = true;
      });
      authenticated = await auth.authenticate(
        localizedReason: 'Scan your fingerprint to authenticate',
      );
      return authenticated;
      setState(() {
        _isAuthenticating = false;
      });
    } on PlatformException catch (e) {
      print(e);
    }
    if (!mounted) return false;
    return false;
  }

  _biometricsSettingsChanged(v, _ViewModel vm) async {
    if (v) {
      var idChecked = await _authenticate();
      if (idChecked) {
        vm.createAccessToken({"name": "Mobile App: ${vm.user?.uuid}"});
        setState(() {
          _biometricEnabled = true;
        });
        return;
      }
    }
    showDialog(
        context: context,
        builder: (BuildContext context) => CupertinoAlertDialog(
              title: new Text("Important Note"),
              content: new Text(
                  "By disabling ${getBiometricLabel(_availableBiometrics!.first)} you will be signed out and redirected to the signin page. You will have to enter your credentials again to continue using the app."),
              actions: <Widget>[
                CupertinoDialogAction(
                  child: Text("Cancel"),
                  onPressed: () => Navigator.of(context).pop(),
                ),
                CupertinoDialogAction(
                  isDefaultAction: true,
                  child: Text("Disable"),
                  onPressed: () {
                    setState(() {
                      _biometricEnabled = false;
                    });
                    Navigator.of(context).pop();
                    vm.deleteAccessToken(vm.biometricAccessToken?.id);
                    vm.removeBiometricAccessToken();
                    vm.clearTokenNetwork();
                  },
                ),
              ],
            ));
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    print(vm.accessTokens);
    return ListView(
      children: [
        Padding(
          padding: const EdgeInsets.all(12.0),
          child: Column(
            children: [
              if (_availableBiometrics != null &&
                  _availableBiometrics!.length > 0)
                Column(
                  children: [
                    Divider(),
                    ListTile(
                      leading: getBiometricIcon(_availableBiometrics!.first),
                      title:
                          Text(getBiometricLabel(_availableBiometrics!.first)),
                      trailing: Switch(
                          value: _biometricEnabled,
                          onChanged: (v) => _biometricsSettingsChanged(v, vm)),
                    ),
                    Divider(),
                  ],
                )
            ],
          ),
        ),
        Padding(
          padding: const EdgeInsets.all(16.0),
          child: PumaButton(
            title: "Signout",
            onPressed: () {
              if (vm.biometricAccessToken == null) {
                vm.signout();
              } else {
                Navigator.pushNamed(context, "/welcome");
              }
            },
          ),
        )
      ],
    );
  }
}

class _ViewModel {
  final User? user;
  final BuiltList<AccessToken>? accessTokens;
  final AccessToken? biometricAccessToken;
  final Function createAccessToken;
  final Function deleteAccessToken;
  final Function removeBiometricAccessToken;
  final Function setBiometricAccessToken;
  final Function clearTokenNetwork;
  final Function signout;

  _ViewModel(
    this.user,
    this.accessTokens,
    this.biometricAccessToken,
    this.createAccessToken,
    this.deleteAccessToken,
    this.removeBiometricAccessToken,
    this.setBiometricAccessToken,
    this.clearTokenNetwork,
    this.signout,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
      store.state.accountKitState.userState.user,
      store.state.accountKitState.accessTokenState.accessTokens,
      store.state.accountKitState.accessTokenState.biometricAccessToken,
      (body) => store.dispatch(CreateAccessTokenAction(body)),
      (id) => store.dispatch(DeleteAccessTokenAction(id)),
      () => store.dispatch(RemoveBiometricAccessToken()),
      (biometricAccessToken) =>
          store.dispatch(SetBiometricAccessToken(biometricAccessToken)),
      () => store.dispatch(ClearTokenNetworkAction()),
      () => store.dispatch(SignOutAction(store.state.networkState.token)),
    );
  }
}
