import 'dart:async';

import 'package:account_kit/entities/accessToken/accessToken.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:account_kit/ui/views/signinForm.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:local_auth/local_auth.dart';
import 'package:puma_network/redux/network/networkAction.dart';
import 'package:redux/redux.dart';

class SigninContainer<T extends VirtualAccountState> extends StatefulWidget {
  final Widget? header;
  final Widget? footer;
  final Widget? illustration;
  final bool plEnabled;
  SigninContainer({
    this.header,
    this.footer,
    this.illustration,
    this.plEnabled = true,
  });
  @override
  _SigninContainerState createState() => _SigninContainerState<T>();
}

class _SigninContainerState<T extends VirtualAccountState>
    extends State<SigninContainer> {
  final String title = "Signin";
  final LocalAuthentication auth = LocalAuthentication();
  bool _canCheckBiometrics = false;
  List<BiometricType>? _availableBiometrics;
  bool _isAuthenticating = false;

  void initState() {
    _checkBiometrics();
    _getAvailableBiometrics();
  }

  Future<void> _checkBiometrics() async {
    bool canCheckBiometrics = false;
    try {
      canCheckBiometrics = await auth.canCheckBiometrics;
    } on PlatformException catch (e) {
      print(e);
    }
    if (!mounted) return;

    setState(() {
      _canCheckBiometrics = canCheckBiometrics;
    });
  }

  Future<void> _getAvailableBiometrics() async {
    List<BiometricType>? availableBiometrics;
    try {
      availableBiometrics = await auth.getAvailableBiometrics();
    } on PlatformException catch (e) {
      print(e);
    }
    if (!mounted) return;

    setState(() {
      _availableBiometrics = availableBiometrics;
    });
  }

  Future<bool> _authenticate() async {
    bool authenticated = false;
    try {
      setState(() {
        _isAuthenticating = true;
      });
      authenticated = await auth.authenticate(
        localizedReason: 'Scan your fingerprint to authenticate',
      );
      return authenticated;
      setState(() {
        _isAuthenticating = false;
      });
    } on PlatformException catch (e) {
      print(e);
    }
    if (!mounted) return false;
    return false;
  }

  void _cancelAuthentication() {
    auth.stopAuthentication();
  }

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      converter: (Store<T> store) => _ViewModel.create(store),
      builder: (BuildContext context, _ViewModel vm) =>
          _createBody(context, vm),
    );
  }

  _getBiometricsPossibilities(AccessToken biometricAccessToken, _ViewModel vm) {
    List<Widget> children = [];

    if (this._canCheckBiometrics &&
        this._availableBiometrics != null &&
        this._availableBiometrics!.length > 0) {
      for (var availableBiometric in _availableBiometrics!) {
        var icon;

        switch (availableBiometric) {
          case BiometricType.face:
            icon = Image.asset(
              'lib/assets/faceid.png',
              package: "account_kit",
              width: 25,
            );
            break;
          case BiometricType.fingerprint:
            icon = Image.asset(
              'lib/assets/fingerprint.png',
              package: "account_kit",
              width: 25,
            );
            break;
          case BiometricType.iris:
            icon = Image.asset(
              'lib/assets/iris.png',
              package: "account_kit",
              width: 25,
            );
            break;
          default:
            icon = Image.asset(
              'lib/assets/fingerprint.png',
              package: "account_kit",
              width: 25,
            );
        }
        children.add(GestureDetector(
          onTap: _isAuthenticating
              ? _cancelAuthentication
              : () => _signinWithBiometrics(vm),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              icon,
              Padding(
                padding: const EdgeInsets.only(left: 12.0),
                child: Text(biometricAccessToken.userName),
              )
            ],
          ),
        ));
      }
    }
    return Column(
      children: children,
    );
  }

  _signinWithBiometrics(_ViewModel vm) async {
    bool authenticated = await _authenticate();
    if (authenticated) {
      vm.setTokenNetworkAction(vm.biometricAccessToken!.token);
      Timer(Duration(milliseconds: 500), () => vm.getCurrentUser());
      return;
    } else {
      return;
    }
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    return Padding(
      padding: const EdgeInsets.only(top: 40.0),
      child: ListView(
        children: [
          Stack(
            children: [
              if (widget.header != null)
                Positioned(
                    left: 0.0,
                    top: 0.0,
                    child: new Container(
                        width: MediaQuery.of(context).size.width,
                        decoration:
                            new BoxDecoration(color: Colors.grey.shade300),
                        child: widget.header)),
              if (widget.footer != null)
                Positioned(
                    left: 0.0,
                    bottom: 0.0,
                    child: new Container(
                        width: MediaQuery.of(context).size.width,
                        decoration:
                            new BoxDecoration(color: Colors.grey.shade300),
                        child: widget.footer)),
              Column(
                mainAxisAlignment: MainAxisAlignment.center,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  Padding(
                    padding: const EdgeInsets.all(20.0),
                    child: Center(
                      child: Container(
                        width: MediaQuery.of(context).size.width - 40 > 600
                            ? 600
                            : MediaQuery.of(context).size.width - 40,
                        child: Wrap(
                          alignment: WrapAlignment.center,
                          runSpacing: 10.0,
                          children: [
                            widget.illustration ?? Text(""),
                            (SigninForm<T>(plEnabled: widget.plEnabled)),
                          ],
                        ),
                      ),
                    ),
                  ),
                  if (vm.biometricAccessToken != null)
                    Column(
                      children: [
                        Divider(),
                        _getBiometricsPossibilities(
                          vm.biometricAccessToken!,
                          vm,
                        ),
                        Divider(),
                      ],
                    ),
                ],
              ),
            ],
          ),
        ],
      ),
    );
  }
}

class _ViewModel {
  final AccessToken? biometricAccessToken;
  final Function setTokenNetworkAction;
  final Function getCurrentUser;

  _ViewModel(
    this.biometricAccessToken,
    this.setTokenNetworkAction,
    this.getCurrentUser,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
        store.state.accountKitState.accessTokenState.biometricAccessToken,
        (token) => store.dispatch(SetTokenNetworkAction(token)),
        () => store.dispatch(GetUserAction()));
  }
}
